/*
 * File:     wx_stdev.h
 * Purpose:  Standard wxWindows event classes
 *
 *                       wxWindows 1.50
 * Copyright (c) 1993 Artificial Intelligence Applications Institute,
 *                   The University of Edinburgh
 *
 *                     Author: Julian Smart
 *                       Date: 7-9-93
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice, author statement and this permission
 * notice appear in all copies of this software and related documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESS,
 * IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL THE ARTIFICIAL INTELLIGENCE APPLICATIONS INSTITUTE OR THE
 * UNIVERSITY OF EDINBURGH BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF
 * DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


#ifndef wxb_stdevh
#define wxb_stdevh

#include "common.h"
#include "wx_obj.h"
#include "wx_sysev.h"

#define EVENT_TYPES_FIRST 10000

/*
 * Command event types
 *
 */

#define wxEVENT_TYPE_BUTTON_COMMAND              (EVENT_TYPES_FIRST + 1)
#define wxEVENT_TYPE_CHECKBOX_COMMAND            (EVENT_TYPES_FIRST + 2)
#define wxEVENT_TYPE_CHOICE_COMMAND              (EVENT_TYPES_FIRST + 4)
#define wxEVENT_TYPE_LISTBOX_COMMAND             (EVENT_TYPES_FIRST + 5)
#define wxEVENT_TYPE_TEXT_COMMAND                (EVENT_TYPES_FIRST + 7)
#define wxEVENT_TYPE_MULTITEXT_COMMAND           (EVENT_TYPES_FIRST + 8)
#define wxEVENT_TYPE_MENU_COMMAND                (EVENT_TYPES_FIRST + 9)
#define wxEVENT_TYPE_SLIDER_COMMAND              (EVENT_TYPES_FIRST + 10)
#define wxEVENT_TYPE_RADIOBOX_COMMAND            (EVENT_TYPES_FIRST + 11)

/*
 * Mouse event types
 *
 */

#define wxEVENT_TYPE_LEFT_DOWN                   (EVENT_TYPES_FIRST + 30)
#define wxEVENT_TYPE_LEFT_UP                     (EVENT_TYPES_FIRST + 31)
#define wxEVENT_TYPE_MIDDLE_DOWN                 (EVENT_TYPES_FIRST + 32)
#define wxEVENT_TYPE_MIDDLE_UP                   (EVENT_TYPES_FIRST + 33)
#define wxEVENT_TYPE_RIGHT_DOWN                  (EVENT_TYPES_FIRST + 34)
#define wxEVENT_TYPE_RIGHT_UP                    (EVENT_TYPES_FIRST + 35)
#define wxEVENT_TYPE_MOTION                      (EVENT_TYPES_FIRST + 36)

/*
 * Character input event type
 *
 */
#define wxEVENT_TYPE_CHAR                        (EVENT_TYPES_FIRST + 50)

// Item or menu event class
class wxCommandEvent: public wxEvent
{
 public:
  char *commandString; // String event argument
  int commandInt;      // Integer event argument
  long extraLong;      // Additional information (e.g. select/deselect)
  char *labelString;   // The label of the item
  char *clientData;    // Arbitrary client data
  wxCommandEvent(WXTYPE commandType);
  ~wxCommandEvent(void) {}

  /*
   * Accessors dependent on context
   *
   */

  // Get listbox/choice client data  
  inline virtual char *GetClientData() { return clientData; }

  // Get listbox selection if single-choice
  inline virtual int GetSelection() { return commandInt; }

  // Get listbox/choice selection string
  inline virtual char *GetString() { return commandString; }

  // Get checkbox value
  inline virtual Bool Checked() { return (Bool)commandInt; }

  // TRUE if the listbox event was a selection.
  inline virtual Bool IsSelection() { return (Bool)extraLong; }

  Bool ReadEvent(istream&);
  Bool WriteEvent(ostream&);
};

// Mouse event class
class wxMouseEvent: public wxEvent
{
 public:
  float x;
  float y;
  Bool leftDown;
  Bool middleDown;
  Bool rightDown;

  Bool controlDown;
  Bool shiftDown;
  Bool altDown;
  
  wxMouseEvent(WXTYPE mouseType);

  // Was it a button event?
  virtual Bool IsButton(void);

  // Was it a down event from button 1, 2 or 3 or any?
  virtual Bool ButtonDown(int but = -1);

  // Was the given button changing state?
  virtual Bool Button(int but);

  // Find state of shift/control keys
  virtual Bool ControlDown(void);
  virtual Bool ShiftDown(void);

  // Find which event was just generated
  virtual Bool LeftDown(void);
  virtual Bool MiddleDown(void);
  virtual Bool RightDown(void);
  virtual Bool LeftUp(void);
  virtual Bool MiddleUp(void);
  virtual Bool RightUp(void);

  // Find the current state of the mouse buttons (regardless
  // of current event type)
  virtual Bool LeftIsDown(void);
  virtual Bool MiddleIsDown(void);
  virtual Bool RightIsDown(void);

  // True if a button is down and the mouse is moving
  virtual Bool Dragging(void);

  // True if the mouse is moving, and no button is down
  virtual Bool Moving(void);

  // Find the position of the event
  virtual void Position(float *x, float *y);

  Bool ReadEvent(istream&);
  Bool WriteEvent(ostream&);
};

// Keyboard input event class
class wxKeyEvent: public wxEvent
{
 public:
  long keyCode;
  Bool controlDown;
  Bool shiftDown;
  Bool altDown;
  
  wxKeyEvent(WXTYPE keyType);

  virtual Bool ControlDown(void);
  virtual Bool ShiftDown(void);
  virtual long KeyCode(void);

  Bool ReadEvent(istream&);
  Bool WriteEvent(ostream&);
};

#endif // wxb_stdevh

